﻿#ifndef HGL_UNICODE_BLOCKS_INCLUDE
#define HGL_UNICODE_BLOCKS_INCLUDE

#include<hgl/type/DataType.h>

namespace hgl
{
    enum class UnicodeBlock
    {
        basic_latin=0,
        latin_1_supplement,
        latin_extended_a,
        latin_extended_b,
        ipa_extensions,
        spacing_modifier_letters,
        combining_diacritical_marks,
        greek_coptic,
        cyrillic,
        cyrillic_supplement,
        armenian,
        hebrew,
        arabic,
        syrian,
        arabic_supplement,
        thaana,
        nko,
        samaritan,
        mandaic,
        syriac_supplement,
        arabic_extended_a,
        devanagari,
        bengali,
        gurmukhi,
        gujarati,
        oriya,
        tamil,
        telugu,
        kannada,
        malayalam,
        sinhala,
        thai,
        lao,
        tibetan,
        myanmar,
        georgian,
        hangul_jamo,
        ethiopic,
        ethiopic_supplement,
        cherokee,
        unified_canadian_aboriginal_syllabics,
        ogham,
        runic,
        tagalog,
        hanunoo,
        buhid,
        tagbanwa,
        khmer,
        mongolian,
        unified_canadian_aboriginal_syllabics_extended,
        limbu,
        tai_le,
        new_tai_lue,
        khmer_symbols,
        buginese,
        tai_tham,
        combining_diacritical_marks_extended,
        balinese,
        sundanese,
        batak,
        lepcha,
        ol_chiki,
        cyrillic_extended_c,
        georgian_extended,
        sundanese_supplement,
        vedic_extensions,
        phonetic_extensions,
        phonetic_extensions_supplement,
        combining_diacritical_marks_supplement,
        latin_extended_additional,
        greek_extended,
        general_punctuation,
        superscripts_and_subscripts,
        currency_symbols,
        combining_diacritical_marks_for_symbols,
        letterlike_symbols,
        number_forms,
        arrows,
        mathematical_operators,
        miscellaneous_technical,
        control_pictures,
        optical_character_recognition,
        enclosed_alphanumerics,
        box_drawing,
        block_elements,
        geometric_shapes,
        miscellaneous_symbols,
        dingbats,
        miscellaneous_mathematical_symbols_a,
        supplemental_arrows_a,
        braille_patterns,
        supplemental_arrows_b,
        miscellaneous_mathematical_symbols_b,
        supplemental_mathematical_operators,
        miscellaneous_symbols_and_arrows,
        glagolitic,
        latin_extended_c,
        coptic,
        georgian_supplement,
        tifinagh,
        ethiopic_extended,
        cyrillic_extended,
        supplemental_punctuation,
        cjk_radicals_supplement,
        kangxi_radicals,
        ideographic_description_characters,
        cjk_symbols_and_punctuation,
        hiragana,
        katakana,
        bopomofo,
        hangul_compatibility_jamo,
        kanbun,
        bopomofo_extended,
        cjk_strokes,
        katakana_phonetic_extensions,
        enclosed_cjk_letters_and_months,
        cjk_compatibility,
        cjk_unified_ideographs_extension_a,
        yijing_hexagram_symbols,
        cjk_unified_ideographs,
        yi_syllables,
        yi_radicals,
        lisu,
        old_lisu_alphabet,
        ciryllic_extended_B,
        bamum,
        modifier_tone_letters,
        latin_extended_d,
        syloty_nagri,
        indian_numeric_character,
        phags_pa,
        saurashtra,
        devanagari_extended_characters,
        kayah_li,
        rejang,
        hangul,
        javanese_alphabet,
        myanmar_extended_b,
        cham_alphabet,
        burmese_alphabet_extention_a,
        tai_viet_alphabet,
        meitei_language_extensions,
        ethiopic_extended_a,
        latin_extended_e,
        cherokee_supplement,
        meitei_language,
        hangul_syllables,
        hangul_jamo_extended_b,
        high_surrogates,
        high_private_use_surrogates,
        low_surrogates,
        private_use_area,
        cjk_compatibility_ideographs,
        alphabetic_presentation_forms,
        arabic_presentation_forms_a,
        variation_selectors,
        vertical_forms,
        combining_half_marks,
        cjk_compatibility_forms,
        small_form_variants,
        arabic_presentation_forms_b,
        halfwidth_and_fullwidth_forms,
        specials,

        //utf16 ^^^^^^^^^^^

        //utf32 vvvvvvvvvvv

        linear_b_syllabary,
        linear_b_ideograms,
        aegean_numbers,
        ancient_greek_numbers,
        ancient_symbols,
        phaistos_disc,
        lycian,
        carian,
        coptic_epact_numbers,
        old_italic,
        gothic,
        old_permic,
        ugaritic,
        old_persian,
        deseret,
        shavian,
        osmanya,
        osage,
        elbasan,
        caucasian_albanian,
        linear_a,
        cypriot_syllabary,
        imperial_aramaic,
        palmyrene,
        nabataean,
        hatran,
        phoenician,
        lydian,
        meroitic_hieroglyphs,
        meroitic_cursive,
        kharoshthi,
        old_south_arabian,
        old_north_arabian,
        manichaean,
        avestan,
        inscriptional_parthian,
        inscriptional_pahlavi,
        psalter_pahlavi,
        old_turkic,
        old_hungarian,
        hanifi_rohingya,
        rumi_numeral_symbols,
        yezidi,
        old_sogdian,
        sogdian,
        chorasmian,
        elymaic,
        brahmi,
        kaithi,
        sora_sompeng,
        chakma,
        mahajani,
        sharada,
        sinhala_archaic_numbers,
        khojki,
        multani,
        khudawadi,
        grantha,
        newa,
        tirhuta,
        siddham,
        modi,
        mongolian_supplement,
        takri,
        ahom,
        dogra,
        warang_citi,
        dives_akuru,
        nandinagari,
        zanabazar_square,
        soyombo,
        pau_cin_hau,
        bhaiksuki,
        marchen,
        masaram_gondi,
        gunjala_gondi,
        makasar,
        lisu_supplement,
        tamil_supplement,
        cuneiform,
        cuneiform_numbers_and_punctuation,
        early_dynastic_cuneiform,
        egyptian_hieroglyphs,
        egyptian_hieroglyph_format_controls,
        anatolian_hieroglyphs,
        bamum_supplement,
        mro,
        bassa_vah,
        pahawh_hmong,
        medefaidrin,
        miao,
        ideographic_symbols_and_punctuation,
        tangut,
        tangut_components,
        khitan_small_script,
        tangut_supplement,
        kana_supplement,
        kana_extended_a,
        small_kana_extension,
        nushu,
        duployan,
        shorthand_format_controls,
        byzantine_musical_symbols,
        musical_symbols,
        ancient_greek_musical_notation,
        mayan_numerals,
        tai_xuan_jing_symbols,
        counting_rod_numerals,
        mathematical_alphanumeric_symbols,
        sutton_sign_writing,
        glagolitic_supplement,
        nyiakeng_puachue_hmong,
        wancho,
        mende_kikakui,
        adlam,
        indic_siyaq_numbers,
        ottoman_siyaq_numbers,
        arabic_mathematical_alphabetic_symbols,
        mahjong_tiles,
        domino_tiles,
        playing_cards,
        enclosed_alphanumeric_supplement,
        enclosed_ideographic_supplement,
        miscellaneous_symbols_and_pictographs,
        emoticons,
        ornamental_dingbats,
        transport_and_map_symbols,
        alchemical_symbols,
        geometric_shapes_extended,
        supplemental_arrows_c,
        supplemental_symbols_and_pictographs,
        chess_symbols,
        symbols_and_pictographs_extended_a,
        symbols_for_legacy_computing,
        cjk_unified_ideographs_extension_b,
        cjk_unified_ideographs_extension_c,
        cjk_unified_ideographs_extension_d,
        cjk_unified_ideographs_extension_e,
        cjk_unified_ideographs_extension_f,
        cjk_compatibility_ideographs_supplement,
        cjk_unified_ideographs_extension_g,
        tags,
        variation_selectors_supplement,
        supplementary_private_use_area_a,
        supplementary_private_use_area_b,

        Error=0xFFFF,

        ENUM_CLASS_RANGE(basic_latin,supplementary_private_use_area_b)
    };//

    /**
     * 寻找字符属于那一个Unicode块
     */
    const UnicodeBlock IndexOfUnicodeBlock(const u32char ch);

    /**
     * 判断字符是否属于当前Unicode块
     */
    bool IsInUnicodeBlock(const UnicodeBlock &type,const u32char ch);

    bool isLatin        (const u32char ch);     ///判断当前字符是否是拉丁字符
    bool isCJK          (const u16char ch);     ///判断当前字符是否是CJK字符
    bool isCJK          (const u32char ch);     ///判断当前字符是否是CJK字符
    bool isEmoji        (const u32char ch);     ///判断当前字符是否是表情符号    
    bool isPunctuation  (const u32char ch);     ///判断当前字符是否是标点符号
}//namespace hgl
#endif//HGL_UNICODE_BLOCKS_INCLUDE
